// Author: Adrian Huang
// Email: sanyuexx@hotmail.com
// Weblog: http://www.cnblogs.com/dah/
// 2007.10

package fxunit
{
	import flash.events.EventDispatcher;
	import flash.utils.getQualifiedClassName;
	
	import fxunit.reflect.ClassInfo;
	import fxunit.reflect.MetadataInfo;
	import fxunit.reflect.MethodInfo;
	import fxunit.reflect.typeinfo;
	
	// Event元数据让你在键入addEventListener后会有相应的Event类型提示
	[Event(name="passed", type="fxunit.TestEvent")]
	[Event(name="failed", type="fxunit.TestEvent")]
	public class TestRunner extends EventDispatcher
	{
		private var _testClass:Class;
		
		// TestRunner只需要一个Class对象
		public function TestRunner(testClass:Class)
		{
			_testClass = testClass;
		}
		
		public function runTests():void
		{
			var testObj:Object = new _testClass();
			// 获取类型信息,这个方法在fxunit.reflect包中
			var ci:ClassInfo = typeinfo(_testClass);
			
			for each(var m:Object in ci.methods)
			{
				var mi:MethodInfo = MethodInfo(m);
				// 检查其Metadata, 验证是否是一个Test方法
				if (isTestMethod(mi))
				{
					// 获取其Function对象
					var method:Function = testObj[mi.name];
					// 检查其Metadata, 是否期望异常
					var expectedError:String = getExpectedError(mi);
					try
					{
						// 调用! 没有异常即通过测试
						method.call(testObj);
						dispatchTestEvent(TestEvent.PASSED, mi);
					}
					catch (ae:AssertError)
					{
						// 断言异常, 测试失败
						dispatchTestEvent(TestEvent.FAILED, mi, ae);
					}
					catch (e:Error)
					{
						// 如果期望异常则检查是否类型相同
						if (expectedError != null)
						{
							var eName:String = getQualifiedClassName(e);
							if (eName == expectedError)
							{
								dispatchTestEvent(TestEvent.PASSED, mi);
							}
							else
							{
								dispatchTestEvent(TestEvent.FAILED, mi, 
										new AssertError("Expected: " + expectedError +
														", but was: " + eName));					
							}
						}
						else // 否则测试失败
						{
							dispatchTestEvent(TestEvent.FAILED, mi, e);
						}						
					}
				}
			}	
		}
		
		protected function dispatchTestEvent(type:String, mi:MethodInfo, e:Error = null)
		{
			dispatchEvent(new TestEvent(type, mi.name, e ? e.message : null));
		}
		
		public static const TEST_METADATA = "Test";
		public static const EXPECTED_ERROR_ARG = "expectedError";
		
		// 检查是否有名为Test的Metadata
		protected function isTestMethod(mi:MethodInfo):Boolean
		{
			for each (var obj:Object in mi.metadata)
			{
				var mdi:MetadataInfo = MetadataInfo(obj);
				if (mdi.name == TEST_METADATA)
				{
					return true;
				}
			}			
			return false;
		}
		
		protected function getExpectedError(mi:MethodInfo):String
		{			
			for each (var obj:Object in mi.metadata)
			{
				var mdi:MetadataInfo = MetadataInfo(obj);
				if (mdi.name == TEST_METADATA && mdi.args.contains(EXPECTED_ERROR_ARG))
				{
					// 如果包含expectedError参数, 则返回参数值
					return String(mdi.args.geti(EXPECTED_ERROR_ARG));
				}
			}		
			return null;
		}
	}
}